function [g,main_gdata,f1lin_p,f2lin_p,z1lin_p,z2lin_p] = autoprox_2d(f1,f2,f1f2_vars,x1b,x2b,f1nlin,f2nlin,y1nlin,y2nlin)
% PWA approximation of the product f1*f2

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

% checking the existence of the required features
[ret_value_fm,ret_value_s]=autoprox_have_fmincon_and_sym;

if isa(f1, 'sym')
    f1 = matlabFunction(f1);
end
if isa(f2, 'sym')
    f2 = matlabFunction(f2);
end

syms x

[f1lin, f1lin_p] = autoprox_1d(f1, f1f2_vars(1), x1b, f1nlin);
[f2lin, f2lin_p] = autoprox_1d(f2, f1f2_vars(2), x2b, f2nlin);
[zb_sum, zb_diff] = autoprox_getminmax(f1lin, x1b, f2lin, x2b);
[z1lin, z1lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_sum, y1nlin);
[z2lin, z2lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_diff, y2nlin);

g = @(x1,x2) (z1lin(f1lin(x1)+f2lin(x2)) - ...
    z2lin(f1lin(x1)-f2lin(x2)) );
g_data = { f1lin_p{1}, f2lin_p{1}, z1lin_p{1}, z2lin_p{1}};

main_gdata = cell(1,4);
main_gdata{1} = g_data;
main_gdata{2} = f1f2_vars;
main_gdata{3} = x1b;
main_gdata{4} = x2b;










