function [cost, grad] = fmincon_cost_int(x, Jf, ns, xbounds, grad_f)

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

if nargin == 0
    cost = 0;
    return
end
% number of points for numeric integration
N_points = 5e2;

if size(x, 2) ~= 1
    % input is the matrix of the form [ [r; NaN] a b ]
    x = [x(1:ns-1, 1); x(:, 2); x(:, 3)];
end

x = real(x);
r = x(1:ns-1);
a = x(ns:2*ns-1);
b = x(2*ns:end);

r = [xbounds(1); r; xbounds(2)];

numeric_integration = (nargin(Jf) == 3);

cost = 0;
for i = 1:ns
    if numeric_integration
        z = linspace(r(i), r(i+1), N_points);
        % numerical integration by the rectangle rule, slow version:
        % d = 0;
        % for j = 1:N_points-1
        %    d = d + (x(j+1)-x(j))*Jf((x(j)+x(j+1))/2, a(i), b(i));
        % end
        
        % numerical integration by the rectangle rule, vectorized:
        % d = diff(z)*Jf((z(1:end-1)+z(2:end))/2, a(i), b(i))';
        
        % numerical integration by the trapezoidal rule, vectorized:
        d = diff(z)*(Jf(z(1:end-1), a(i), b(i)) + Jf(z(2:end), a(i), b(i)))'/2;
        
        cost = cost + real(d);
    else
        cost = cost + real(Jf(a(i), b(i), r(i), r(i+1)));
    end
end
grad = [];
