function [bsum, bdiff] = autoprox_getminmax(f1, b1, f2, b2)
% Computes the minimum/maximum values of f1+f2 and f1-f2

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

N = 1e3;
x1 = linspace(b1(1), b1(2), N);
x2 = linspace(b2(1), b2(2), N);

f1x = f1(x1);
f2x = f2(x2);

f1max = max(f1x);
f1min = min(f1x);
f2max = max(f2x);
f2min = min(f2x);

sum_max = max([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);
sum_min = min([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);

diff_max = max([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);
diff_min = min([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);

bsum = [sum_min sum_max];
bdiff = [diff_min diff_max];
