function [XS, J1, Jinf] = autoprox_greedy(f, xbounds, N_points, doplot, XS_init)
% Compute initial guesses by a greedy approach

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

remove_init = false;
if nargin < 5
    XS_init = xbounds;
    remove_init = true;
end
if isempty(XS_init)
    XS_init = mean(xbounds);
    remove_init = true;
end

% derivation of f(x)
syms x
df = diff(f(x), x);
dff = matlabFunction(df);

syms xs
aprx = subs(df, xs)*(x-xs)+subs(f, xs);
aprx = matlabFunction(aprx, 'vars', {'x', 'xs'});

if N_points > 1
    
    XS = XS_init;
    for i = 1:N_points
        [e1, eInf, eInf_x] = eval_linearization(f, aprx, xbounds, XS);
        XS = [XS eInf_x];
    end
    % % XS
    if remove_init
        XS = XS(length(XS_init)+1:end);
    end
    [J1, Jinf] = eval_linearization(f, aprx, xbounds, XS);

else
    % N_points == 1
    J1 = 0; Jinf = 0;
    XS = mean(XS_init);
end

[J1 Jinf];

if doplot
    plot_linearizations(f, xbounds, XS);
end

if nargin(dff) == 0
    XS = linspace(xbounds(1), xbounds(2), N_points);
else
    XS = sort(XS(:)');
end
XS = points2regions(sort(XS), f);


%-------------------------------------------------------------------------
function [e1, eInf, eInf_x] = eval_linearization(f, aprx, xbounds, xs_others)

n_eval_points = 1e4;

xs_others = xs_others(:); xs_others = xs_others';
n_others = length(xs_others);
x_eval = linspace(min(xbounds), max(xbounds), n_eval_points);

% original function values
f_values = f(x_eval)';
%f_values = subs(f, x, x_eval)';

% values of individual linearizations
q1 = repmat(x_eval(:), n_others, 1);
q2 = kron(xs_others(:), ones(n_eval_points, 1));
%other_values = reshape(subs(aprx, {x, xs}, {q1, q2}), n_eval_points, n_others);
other_values = reshape(aprx(q1, q2), n_eval_points, n_others);

% best linearization values
difference = abs(repmat(f_values, 1, n_others) - other_values);
best_linearization = min(difference, [], 2);

% linearization errors
e1 = sum(best_linearization) / n_eval_points;
[eInf, eInf_x_idx] = max(best_linearization);
eInf_x = x_eval(eInf_x_idx(1));
