function apprx_data = approximation_ND(fun,vars,xbmin,xbmax,Nlin_x,Nlin_u,arg)
% PWA approximation of the product f1*f2*f3......fN where Nmax = 10

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

%%% splitting the main function to subfunctions %%% 
[L,V] = split(fun,vars);
[subL,subV] = separate_functions(L,V);
f = obtain_subfunctions(subL,subV);

%%% checking, whether the the input functions are symbolic %%% 
for i1 = 1:length(f{1})
    if isa(sym(f{1}{i1}), 'sym')
        f1{1}{i1} = matlabFunction(sym(f{1}{i1}));
    end
end

if(length(xbmin) ~= length(f{1}))
    error('Array of minimal bounds values contains too many or not enough arguments');
end

if(length(xbmax) ~= length(f{1}))
    error('Array of maximal bounds values contains too many or not enough arguments');
end

if(length(Nlin_x) ~= length(f{1}))
    error('Array of number of approximation points contains too many or not enough arguments');
end

if(length(Nlin_u) ~= 2*(length(Nlin_x)-1))
    error('Array of number of approximation points contains too many or not enough arguments');
end

%%% memory allocation for function handles and datas, %%%
%%% obtained from the approximation of the original functions %%%  

flin_p = cell(1,length(f{1}));
flin = cell(1,length(f{1}));
zlin = cell(1,2*(length(f{1})-1));
zlin_p = cell(1,2*(length(f{1})-1));

%%% approximation of the 1D funcions %%%
for i = 1:length(flin_p)
    [f1lin, f1lin_p] = autoprox_1d(f1{1}{i}, vars(i), [xbmin(i) xbmax(i)], Nlin_x(i));
    flin_p{i} = f1lin_p{1};
    flin{i} = f1lin;
end

%%% obtain the bounds for the approximations %%%
[zb_sum, zb_diff] = autoprox_getminmax(flin{1}, [xbmin(1) xbmax(1)], flin{2}, [xbmin(2) xbmax(2)]);

syms x

%%% approximation of the quadratic functional dependences %%%
[z1lin, z1lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_sum, Nlin_u(1));
[z2lin, z2lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_diff, Nlin_u(2));

zlin{1} = z1lin; zlin{2} = z2lin;
zlin_p{1} = z1lin_p{1}; zlin_p{2} = z2lin_p{1};

%%% evaluation of the approximated functions on the given arguments %%%
f_hod = cell(1,length(f{1}));
for i = 1:length(f_hod)
    f_hod{i} = eval_multilin(flin_p{i},linspace(xbmin(i),xbmax(i),3));
end

%%% evaluation of the all possible combinations of input arguments %%%
Uarg = cell(1,2);
Uarg{1} = f_hod{1};
Uarg{2} = f_hod{2};

Ucombs = mpt_allcombs(Uarg);
f1z = Ucombs(:,1);
f2z = Ucombs(:,2);

%%% obtaining the good_values %%%
y1 = eval_multilin(zlin_p{1},(f1z+f2z));
y2 = eval_multilin(zlin_p{2},(f1z-f2z)); 
z = y1 - y2;

%%% for dimension N > 3 %%%
subscript = 3;
for n = 3:length(f{1})

%%% finding the minimal and maximal function values %%%
f2min=min(z);
f2max=max(z);
 
f1min=min(f_hod{n});
f1max=max(f_hod{n});

%%% obtaining the boundaries for apporximation of the kvadratic functions %%% 
u1max = max([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);
u1min = min([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);

u2max = max([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);
u2min = min([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);

zb_sum = [u1min u1max];
zb_diff = [u2min u2max];

%%% approximation of the quadratic functions %%%
[z1lin, z1lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_sum, Nlin_u(subscript));
[z2lin, z2lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_diff, Nlin_u(subscript+1));

%%% data storage for the slopes and affine terms for 
zlin{subscript} = z1lin; zlin{subscript+1} = z2lin;
zlin_p{subscript} = z1lin_p{1}; zlin_p{subscript+1} = z2lin_p{1};

%%% evaluation of all possible combinations of input arguments %%%
Uarg = cell(1,2);
Uarg{1} = z';
Uarg{2} = f_hod{n};

Ucombs = mpt_allcombs(Uarg);
fzz = Ucombs(:,1);
f3z = Ucombs(:,2);

%%% approximation of the quadratic functions %%%
y1 = eval_multilin(z1lin_p{1},(fzz+f3z));
y2 = eval_multilin(z2lin_p{1},(fzz-f3z));
z = y1-y2;

subscript = subscript+2;
end

apprx_data = {flin,flin_p,zlin,zlin_p,Nlin_x,Nlin_u,subV,xbmin,xbmax};




