function main_datas = autoprox_nd_sum(fun,vars,xbmin,xbmax,Nlinx_1,Nlinu_1)
                        
% PWA approximation of the product f1*f2*f3......fN where Nmax = 10

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

main_datas = cell(1,length(Nlinx_1));

if ~iscell(vars)
    cv = {};
    for i = 1:length(vars)
        cv{end+1} = vars(i);
    end
    vars = cv;
end

[L,V] = split(fun,vars);
[subL,subV] = separate_functions(L,V);
f = obtain_subfunctions(subL,subV);
[xbmin_main,xbmax_main] = obtain_the_bounds(vars,xbmin,xbmax,subL,subV);
xbmin_1 = xbmin_main;
xbmax_1 = xbmax_main;

for n = 1:length(Nlinx_1)
    
    if (length(Nlinx_1{n}) > 1)
        
        %%% checking, whether the the input functions are symbolic %%%
        for i1 = 1:length(f{n})
            if isa(sym(f{n}{i1}), 'sym')
                f1{i1} = matlabFunction(sym(f{n}{i1}));
            end
        end
        
        if(length(xbmin_1{n}) ~= length(f{n}))
            error('Array of minimal bounds values contains too many or not enough arguments');
        end
        
        if(length(xbmax_1{n}) ~= length(f{n}))
            error('Array of maximal bounds values contains too many or not enough arguments');
        end
        
        if(length(Nlinx_1{n}) ~= length(f{n}))
            error('Array of number of approximation points contains too many or not enough arguments');
        end
        
        if(length(Nlinu_1{n}) ~= 2*(length(Nlinx_1{n})-1))
            error('Array of number of approximation points contains too many or not enough arguments');
        end
        
        %%% memory allocation for function handles and datas, %%%
        %%% obtained from the approximation of the original functions %%%
        
        flin_p = cell(1,length(f{n}));
        flin = cell(1,length(f{n}));
        zlin = cell(1,2*(length(f{n})-1));
        zlin_p = cell(1,2*(length(f{n})-1));
        
        %%% approximation of the 1D funcions %%%
        for i = 1:length(flin_p)
            [f1lin, f1lin_p] = autoprox_1d(f1{i}, vars(i), [xbmin_1{n}(i) xbmax_1{n}(i)], Nlinx_1{n}(i));
            flin_p{i} = f1lin_p{1};
            flin{i} = f1lin;
        end
        
        %%% obtain the bounds for the approximations %%%
        [zb_sum, zb_diff] = autoprox_getminmax(flin{1}, [xbmin_1{n}(1) xbmax_1{n}(1)], flin{2}, [xbmin_1{n}(2) xbmax_1{n}(2)]);
        
        %%% approximation of the kvadratic functional dependences %%%
        syms x
        [z1lin, z1lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_sum, Nlinu_1{n}(1));
        [z2lin, z2lin_p] = autoprox_1d(@(x) 1/4*x.^2, x, zb_diff, Nlinu_1{n}(2));
        
        zlin{1} = z1lin; zlin{2} = z2lin;
        zlin_p{1} = z1lin_p{1}; zlin_p{2} = z2lin_p{1};
        
        %%% evaluation of the approximated functions on the given arguments %%%
        f_hod = cell(1,length(f{n}));
        for i = 1:length(f_hod)
            f_hod{i} = eval_multilin(flin_p{i},linspace(xbmin_1{n}(i),xbmax_1{n}(i),3));
        end
        
        %%% evaluation of the all possible combinations of input arguments %%%
        Uarg = cell(1,2);
        Uarg{1} = f_hod{1};
        Uarg{2} = f_hod{2};
        
        Ucombs = mpt_allcombs(Uarg);
        f1z = Ucombs(:,1);
        f2z = Ucombs(:,2);
        
        %%% obtaining the good_values %%%
        y1 = eval_multilin(zlin_p{1},(f1z+f2z));
        y2 = eval_multilin(zlin_p{2},(f1z-f2z));
        z = y1 - y2;
        
        %%% for dimension N > 3 %%%
        subscript = 3;
        for m = 3:length(f{n})
            
            %%% finding the minimal and maximal function values %%%
            f2min=min(z);
            f2max=max(z);
            
            f1min=min(f_hod{m});
            f1max=max(f_hod{m});
            
            %%% obtaining the boundaries for apporximation of the kvadratic functions %%%
            u1max = max([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);
            u1min = min([(f1max+f2max), (f1max+f2min), (f1min+f2max), (f1min+f2min)]);
            
            u2max = max([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);
            u2min = min([(f1max-f2max), (f1max-f2min), (f1min-f2max), (f1min-f2min)]);
            
            zb_sum = [u1min u1max];
            zb_diff = [u2min u2max];
            
            %%% approximation of the kvadratic functions %%%
            syms x
            [z1lin, z1lin_p] = autoprox_1d(@(x) 1/4*x.^2, x,  zb_sum, Nlinu_1{n}(subscript));
            [z2lin, z2lin_p] = autoprox_1d(@(x) 1/4*x.^2, x,  zb_diff, Nlinu_1{n}(subscript+1));
            
            %%% data storage for the slopes and affine terms for vars{1}
            zlin{subscript} = z1lin; zlin{subscript+1} = z2lin;
            zlin_p{subscript} = z1lin_p{1}; zlin_p{subscript+1} = z2lin_p{1};
            
            %%% evaluation of all possible combination of input arguments %%%
            Uarg = cell(1,2);
            Uarg{1} = z';
            Uarg{2} = f_hod{m};
            
            Ucombs = mpt_allcombs(Uarg);
            fzz = Ucombs(:,1);
            f3z = Ucombs(:,2);
            
            %%% approximation of the kvadratic functions %%%
            y1 = eval_multilin(z1lin_p{1},(fzz+f3z));
            y2 = eval_multilin(z2lin_p{1},(fzz-f3z));
            z = y1-y2;
            
            subscript = subscript+2;
        end
        
        apprx_data = {flin,flin_p,zlin,zlin_p,Nlinx_1,Nlinu_1,subV};
        main_datas{n} = apprx_data;
        
    else
        
        [ft, xf]  = autoprox_1d(sym(f{n}), [xbmin_1{n}(1) xbmax_1{n}(1)], Nlinx_1{n}(1));
        apprx_data = {ft,xf,0,0,Nlinx_1,Nlinu_1,subV};
        
        main_datas{n} = apprx_data;
        
    end
    
end


if(nargin < 10)
    main_datas{n+1} = vars;
    main_datas{n+2}{1} = xbmin;
    main_datas{n+2}{2} = xbmax;
    main_datas{n+3} = L;
else
    main_datas{n+1}{1} = vars_state;
    main_datas{n+1}{2} = vars_input;
    main_datas{n+2}{1} = xbmin;
    main_datas{n+2}{2} = xbmax;
    main_datas{n+2}{3} = umin1;
    main_datas{n+2}{4} = umax1;
    main_datas{n+3} = right_side;
end


%-----------------------------------------------------------------
function [xbmin_main,xbmax_main] = obtain_the_bounds(vars,xbmin,xbmax,subL,subV)

L = subL;
positions = cell(1,length(L));

xbmin_main = cell(1,length(L));
xbmax_main = cell(1,length(L));

for i = 1:length(L)
    temp = [];
    for j = 1:length(subV{i})
        for n = 1:length(vars)
            if(subV{i}(j) == vars{n})
               temp = [temp n];
               break;
            end
        end
    end
    positions{i} = temp;
    xbmin_main{i} = xbmin(positions{i});
    xbmax_main{i} = xbmax(positions{i});
end
