close all

% checking the existence of the required features
[ret_value_fm,ret_value_s]=autoprox_have_fmincon_and_sym;

syms z1 z2

% approximate the product f1*f2
f1 = z1^3;
f2 = abs(z2)+0.5*z2^2-sin(z2^3);

% array of symbolic variables(arguments of the functions) 
f1f2_vars = [z1 z2];

% domains of f1 and f2
f1_bounds = [-2, 2.5];
f2_bounds = [-2, 2.5];

% number of approximation segments
f1_segments = 3;
f2_segments = 8;
y1_segments = 3;  % approximation of (f1+f2)^2
y2_segments = 3;  % approximation of (f1-f2)^2

[f_aprx, f_aprx_data] = autoprox_2d(f1, f2, f1f2_vars, f1_bounds, f2_bounds, ...
    f1_segments, f2_segments, y1_segments, y2_segments);

% export the approximation to a HYSDEL file
hysdel_2d(f_aprx_data, 'f1f2_aprx.hys');

% generating the hysdel sim file 
hysdel('f1f2_aprx.hys','f1f2_aprx_sim.m');

% plot the original function and the approximation
f = f1*f2;
fm = matlabFunction(f);

[x1, x2] = meshgrid(linspace(f1_bounds(1), f1_bounds(2), 100), ...
    linspace(f2_bounds(1), f2_bounds(2), 100));
subplot(1, 2, 1);
surf(x1, x2, fm(x1, x2));
axis tight
title('Original function.');
xlabel('z1');ylabel('z2');
subplot(1, 2, 2);
surf(x1, x2, f_aprx(x1, x2));
axis tight
title('Approximation.');
xlabel('z1');ylabel('z2');
