function hysdel_1d(main_xf, fname)
% Converts the PWA approximation of a function of one variable into HYSDEL

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

fw = fopen(fname, 'w');

xf = main_xf{1};
sym_variable = main_xf{2};
var_bounds = main_xf{3};

r_values = xf(1:end-1, 1);
a_values = xf(:, 2);
b_values = xf(:, 3);

fprintf(fw,'SYSTEM f_approx {');
fprintf(fw,'\n\t\t');
fprintf(fw,'INTERFACE {');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'INPUT  { REAL ');
fprintf(fw,'%s',char(sym_variable));
fprintf(fw,'[');
fprintf(fw,'%.4f',var_bounds(1));
fprintf(fw,', ');
fprintf(fw,'%.4f',var_bounds(2));
fprintf(fw,'];');
fprintf(fw,'}');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'OUTPUT { REAL f; }');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'PARAMETER {');

a = {};
b = {};
    for i = 1:size(xf,1)
        j = num2str(i);
        a{i} = strcat('a',j);
        b{i} = strcat('b',j);
    end
r = {};
    for i = 1:size(xf,1)-1
        j = num2str(i);
        r{i} = strcat('r',j);
    end

for i = 1:size(xf,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(a(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',a_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
for i = 1:size(xf,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(b(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',b_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
   
for i = 1:size(xf,1)-1
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(r(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',r_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

fprintf(fw,'}');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'}');
fprintf(fw,'\n\t\t');
fprintf(fw,'IMPLEMENTATION {');
fprintf(fw,'\n\t\t\t AUX {');
fprintf(fw,'\n\t\t\t BOOL ');

d = {};
    for i = 1:size(xf,1)-1
        j = num2str(i);
        d{i} = strcat('d',j);
    end

f = {};

    for i = 1:size(xf,1)
        j = num2str(i);
        f{i} = strcat('f',j);
    end
    f{i+1} = strcat('f','s');

for i = 1:size(xf,1)-1
    fprintf(fw,'%c',char(d(i)));
    if(i ~= size(xf,1)-1)
        fprintf(fw,',');
    end
end
fprintf(fw,';');
fprintf(fw,'\n\t\t\t REAL fs,');

for i = 1:size(xf,1)
    fprintf(fw,'%c',char(f(i)));
    if(i ~= size(xf,1))
        fprintf(fw,',');
    end
end
fprintf(fw,';');
fprintf(fw,'} ');
fprintf(fw,'\n\t\t\t AD {');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(d)
    fprintf(fw,'%c',char(d(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%s',char(sym_variable));
    fprintf(fw,' <= ');
    fprintf(fw,'%c',char(r(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'\n\t\t\t }');
fprintf(fw,'\n\t\t\t DA {');
fprintf(fw,'\n\t\t\t\t ');

fprintf(fw,'f1 = { IF  d1      THEN a1*' );
fprintf(fw,'%s',char(sym_variable));
fprintf(fw,'+b1 };');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(d)-1
    fprintf(fw,'%c',f{i+1});
    fprintf(fw,' = { IF ~');
    fprintf(fw,'%c',char(d(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(d(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(a(i+1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',char(sym_variable));
    fprintf(fw,'+');
    fprintf(fw,'%c',char(b(i+1)));
    fprintf(fw,' };');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%c',f{i+2});
fprintf(fw,' = { IF ~');
fprintf(fw,'%c',char(d(i+1)));
fprintf(fw,'     ');
fprintf(fw,' THEN ');
fprintf(fw,'%c',char(a(i+2)));
fprintf(fw,'*');
fprintf(fw,'%s',char(sym_variable));
fprintf(fw,'+');
fprintf(fw,'%c',char(b(i+2)));
fprintf(fw,' };');
fprintf(fw,'\n\t\t\t }');

fprintf(fw,'\n\t\t\t LINEAR { fs = ');

for i = 1:length(f) - 1
   fprintf(fw,'%c',char(f(i)));
   if(i ~= length(f) - 1)
      fprintf(fw,' + ');
   end
end
fprintf(fw,'; }');
fprintf(fw,'\n\t\t\t ');
fprintf(fw,'OUTPUT { f = fs; }');
fprintf(fw,'\n\t\t }');
fprintf(fw,'\n }');

fclose(fw);
