function hysdel_2d(f_data, fname)
% Converts the PWA approximation a product of two functions into HYSDEL

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

fw = fopen(fname,'w');

f_data1 = f_data{1};
sym_variables = f_data{2};
fbounds_f1 = f_data{3};
fbounds_f2 = f_data{4};

xf1 = f_data1{1}; xf2 = f_data1{2};
xu1 = f_data1{3}; xu2 = f_data1{4};
a_values1 = f_data1{1}(:, 2);
b_values1 = f_data1{1}(:, 3);
r_values1 = f_data1{1}(1:end-1, 1);

a_values2 = f_data1{2}(:, 2);
b_values2 = f_data1{2}(:, 3);
r_values2 = f_data1{2}(1:end-1, 1);

au1_values = f_data1{3}(:, 2);
bu1_values = f_data1{3}(:, 3);
ru1_values = f_data1{3}(1:end-1, 1);

au2_values = f_data1{4}(:, 2);
bu2_values = f_data1{4}(:, 3);
ru2_values = f_data1{4}(1:end-1, 1);

fprintf(fw,'SYSTEM f_approx1x2 {');
fprintf(fw,'\n\t\t');
fprintf(fw,'INTERFACE {');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'INPUT { ');
for i = 1:2
    fprintf(fw,'REAL ');
    fprintf(fw,'%s ',char(sym_variables(i)));
    fprintf(fw,'[ ');
    if(i == 1)
        fprintf(fw,'%.4f',fbounds_f1(1));
        fprintf(fw,', ');
        fprintf(fw,'%.4f',fbounds_f1(2));
    else
        fprintf(fw,'%.4f',fbounds_f2(1));
        fprintf(fw,', ');
        fprintf(fw,'%.4f',fbounds_f2(2));
    end
    fprintf(fw,'];');
    if(i == 1)
        fprintf(fw,'\n\t\t\t\t    ');
    end
end
fprintf(fw,'\n\t\t\t\t   }');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'OUTPUT { REAL f1_f2; }');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'PARAMETER {');

% first function %

a = {};
b = {};
    index = 1;
    for i = 1:size(xf1,1)
        j = num2str(i);
        sub = num2str(index);
        a{i} = strcat('a',sub);
        temp11a = a{i};
        a{i} = strcat(temp11a,'_');
        temp12a = a{i};
        a{i} = strcat(temp12a,j); 
        b{i} = strcat('b',sub);
        temp11b = b{i};
        b{i} = strcat(temp11b,'_');
        temp12b = b{i};
        b{i} = strcat(temp12b,j); 
    end
 
for i = 1:size(xf1,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(a(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',a_values1(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
for i = 1:size(xf1,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(b(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',b_values1(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end    
    
r = {};
     
for i = 1:size(xf1,1)-1
    j = num2str(i);
    sub = num2str(index);
    r{i} = strcat('r',sub);
    temp11a = r{i};
    r{i} = strcat(temp11a,'_');
    temp12a = r{i};
    r{i} = strcat(temp12a,j);
end
    
    
 for i = 1:size(xf1,1)-1
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(r(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',r_values1(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
 end

d = {};
index = 1;
for i = 1:size(xf1,1)-1
    j = num2str(i);
    sub = num2str(index);
    d{i} = strcat('d',sub);
    temp11a = d{i};
    d{i} = strcat(temp11a,'_');
    temp12a = d{i};
    d{i} = strcat(temp12a,j);
end
    
 %%% second function %%%   
 
 a1 = {};
 b1 = {};
 
 index = 2;
 for i = 1:size(xf2,1)
     j = num2str(i);
     sub = num2str(index);
     a1{i} = strcat('a',sub);
     temp21a = a1{i};
     a1{i} = strcat(temp21a,'_');
     temp22a = a1{i};
     a1{i} = strcat(temp22a,j);
     b1{i} = strcat('b',sub);
     temp21b = b1{i};
     b1{i} = strcat(temp21b,'_');
     temp22b = b1{i};
     b1{i} = strcat(temp22b,j);
 end
 
for i = 1:size(xf2,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(a1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',a_values2(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
for i = 1:size(xf2,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(b1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',b_values2(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

r1 = {};

index = 2;
for i = 1:size(xf2,1)-1
    j = num2str(i);
    sub = num2str(index);
    r1{i} = strcat('r',sub);
    temp11a = r1{i};
    r1{i} = strcat(temp11a,'_');
    temp12a = r1{i};
    r1{i} = strcat(temp12a,j);
end

for i = 1:size(xf2,1)-1
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(r1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',r_values2(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

%%%%% parameters for the functions fu1 and fu2 %%%%%%

au1 = {};
bu1 = {};
for i = 1:size(xu1,1)
    j = num2str(i);
    au1{i} = strcat('a','u1');
    temp11a = au1{i};
    au1{i} = strcat(temp11a,'_');
    temp12a = au1{i};
    au1{i} = strcat(temp12a,j);
    bu1{i} = strcat('b','u1');
    temp11b = bu1{i};
    bu1{i} = strcat(temp11b,'_');
    temp12b = bu1{i};
    bu1{i} = strcat(temp12b,j);
end

au2 = {};
bu2 = {};
for i = 1:size(xu1,1)
    j = num2str(i);
    au2{i} = strcat('a','u2');
    temp11a = au2{i};
    au2{i} = strcat(temp11a,'_');
    temp12a = au2{i};
    au2{i} = strcat(temp12a,j);
    bu2{i} = strcat('b','u2');
    temp11b = bu2{i};
    bu2{i} = strcat(temp11b,'_');
    temp12b = bu2{i};
    bu2{i} = strcat(temp12b,j);
end

for i = 1:size(xu1,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(au1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',au1_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
for i = 1:size(xu1,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(bu1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',bu1_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

for i = 1:size(xu2,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(au2(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',au2_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
for i = 1:size(xu2,1)
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(bu2(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',bu2_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

ru = {};
for i = 1:size(xu1,1)-1
    j = num2str(i);
    ru{i} = strcat('r','u1');
    temp11a = ru{i};
    ru{i} = strcat(temp11a,'_');
    temp12a = ru{i};
    ru{i} = strcat(temp12a,j);
end
   
for i = 1:size(xu1,1)-1
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(ru(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',ru1_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end

ru1 = {};
for i = 1:size(xu2,1)-1
    j = num2str(i);
    ru1{i} = strcat('r','u2');
    temp11a = ru1{i};
    ru1{i} = strcat(temp11a,'_');
    temp12a = ru1{i};
    ru1{i} = strcat(temp12a,j);
end
   
for i = 1:size(xu2,1)-1
    fprintf(fw,' REAL ');
    fprintf(fw,'%c',char(ru1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%.4f',ru2_values(i));
    fprintf(fw,';');
    fprintf(fw,'\n');
    fprintf(fw,'\t\t\t\t\t   ');
end
    
fprintf(fw,'}');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'}');
fprintf(fw,'\n\t\t');
fprintf(fw,'IMPLEMENTATION {');
fprintf(fw,'\n\t\t\t AUX {');
fprintf(fw,'\n\t\t\t BOOL ');

d1 = {};
index = 2;
    for i = 1:size(xf2,1)-1
        j = num2str(i);
        sub = num2str(index);
        d1{i} = strcat('d',sub);
        temp21a = d1{i};
        d1{i} = strcat(temp21a,'_');
        temp22a = d1{i};
        d1{i} = strcat(temp22a,j);
    end

for i = 1:size(xf1,1)-1
    fprintf(fw,'%c',char(d(i)));
    if(i ~= size(xf1,1)-1)
        fprintf(fw,',');
    end
end    

fprintf(fw,',');

for i = 1:size(xf2,1)-1
    fprintf(fw,'%c',char(d1(i)));
    if(i ~= size(xf2,1)-1)
		fprintf(fw,',');
    end
end
fprintf(fw,';');
fprintf(fw,'\n\t\t\t REAL ');

f = {};
index = 1;
for i = 1:size(xf1,1)
    j = num2str(i);
    sub = num2str(index);
    f{i} = strcat('f',sub);
    temp21a = f{i};
    f{i} = strcat(temp21a,'_');
    temp22a = f{i};
    f{i} = strcat(temp22a,j);
end

for i = 1:size(xf1,1)
    fprintf(fw,'%c',char(f(i)));
    if(i ~= size(xf1,1))
        fprintf(fw,',');
    end
end

f1 = {};
index = 2;
for i = 1:size(xf2,1)
    j = num2str(i);
    sub = num2str(index);
    f1{i} = strcat('f',sub);
    temp21a = f1{i};
    f1{i} = strcat(temp21a,'_');
    temp22a = f1{i};
    f1{i} = strcat(temp22a,j);
end

fprintf(fw,',');

for i = 1:size(xf2,1)
    fprintf(fw,'%c',char(f1(i)));
    if(i ~= size(xf2,1))
        fprintf(fw,',');
    end
end
fprintf(fw,',');
fprintf(fw,'f1');
fprintf(fw,',');
fprintf(fw,'f2');

fprintf(fw,';\n\t\t\t BOOL ');

du = {};
    for i = 1:size(xu1,1)-1
        j = num2str(i);
        du{i} = strcat('d','u1');
        temp21a = du{i};
        du{i} = strcat(temp21a,'_');
        temp22a = du{i};
        du{i} = strcat(temp22a,j);
    end

du1 = {};
index = 2;
    for i = 1:size(xu2,1)-1
        j = num2str(i);
        sub = num2str(index);
        du1{i} = strcat('d','u2');
        temp21a = du1{i};
        du1{i} = strcat(temp21a,'_');
        temp22a = du1{i};
        du1{i} = strcat(temp22a,j);
    end

for i = 1:size(xu1,1)-1
    fprintf(fw,'%c',char(du(i)));
    if(i ~= size(xu1,1)-1)
        fprintf(fw,',');
    end
end    

fprintf(fw,',');

for i = 1:size(xu2,1)-1
    fprintf(fw,'%c',char(du1(i)));
    if(i ~= size(xu2,1)-1)
        fprintf(fw,',');
    end
end
fprintf(fw,';');
fprintf(fw,'\n\t\t\t REAL ');

fu = {};
for i = 1:size(xu1,1)
    j = num2str(i);
    fu{i} = strcat('f','u1');
    temp21a = fu{i};
    fu{i} = strcat(temp21a,'_');
    temp22a = fu{i};
    fu{i} = strcat(temp22a,j);
end

for i = 1:size(xu1,1)
    fprintf(fw,'%c',char(fu(i)));
    if(i ~= size(xu1,1))
        fprintf(fw,',');
    end
end

fu1 = {};
for i = 1:size(xu2,1)
    j = num2str(i);
    fu1{i} = strcat('f','u2');
    temp21a = fu1{i};
    fu1{i} = strcat(temp21a,'_');
    temp22a = fu1{i};
    fu1{i} = strcat(temp22a,j);
end

fprintf(fw,',');

for i = 1:size(xu2,1)
    fprintf(fw,'%c',char(fu1(i)));
    if(i ~= size(xu2,1))
        fprintf(fw,',');
    end
end

fprintf(fw,',');
fprintf(fw,'fu1');
fprintf(fw,',');
fprintf(fw,'fu2');
fprintf(fw,',');
fprintf(fw,'fc');

fprintf(fw,';');

fprintf(fw,'\n\t\t\t } ');


%----------------------------------------------------------
fprintf(fw,'\n\t\t\t AD {');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(d)
    fprintf(fw,'%c',char(d(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%s',char(sym_variables(1)));
    fprintf(fw,' <= ');
    fprintf(fw,'%c',char(r(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end

for i = 1:length(d1)
    fprintf(fw,'%c',char(d1(i)));
    fprintf(fw,' = ');
    fprintf(fw,'%s',char(sym_variables(2)));
    fprintf(fw,' <= ');
    fprintf(fw,'%c',char(r1(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end

fprintf(fw,'}');
fprintf(fw,'\n\t\t\t DA {');
fprintf(fw,'\n\t\t\t\t ');

fprintf(fw,'f1_1 = {IF  d1_1        THEN a1_1*');
fprintf(fw,'%s',char(sym_variables(1)));
fprintf(fw,'+b1_1');
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(d)-1
    fprintf(fw,'%c',f{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(d(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(d(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(a(i+1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',char(sym_variables(1)));
    fprintf(fw,'+');
    fprintf(fw,'%c',char(b(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%c',f{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(d(i+1)));
fprintf(fw,'     ');
fprintf(fw,'   THEN ');
fprintf(fw,'%c',char(a(i+2)));
fprintf(fw,'*');
fprintf(fw,'%s',char(sym_variables(1)));
fprintf(fw,'+');
fprintf(fw,'%c',char(b(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t ');

fprintf(fw,'f2_1 = {IF  d2_1        THEN a2_1*' );
fprintf(fw,'%s',char(sym_variables(2)));
fprintf(fw,'+b2_1');
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(d1)-1
    fprintf(fw,'%c',f1{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(d1(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(d1(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(a1(i+1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',char(sym_variables(2)));
    fprintf(fw,'+');
    fprintf(fw,'%c',char(b1(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%c',f1{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(d1(i+1)));
fprintf(fw,'     ');
fprintf(fw,'   THEN ');
fprintf(fw,'%c',char(a1(i+2)));
fprintf(fw,'*');
fprintf(fw,'%s',char(sym_variables(2)));
fprintf(fw,'+');
fprintf(fw,'%c',char(b1(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t }');

fprintf(fw,'\n\t\t\t LINEAR { f1 = ');
for i = 1:length(f)
   fprintf(fw,'%c',char(f(i)));
   if(i ~= length(f))
      fprintf(fw,' + ');
   end
end
fprintf(fw,';');

fprintf(fw,'\n\t\t\t\t      f2 = ');
for i = 1:length(f1)
   fprintf(fw,'%c',char(f1(i)));
   if(i ~= length(f1))
      fprintf(fw,' + ');
   end
end
fprintf(fw,'; }');

fprintf(fw,'\n\t\t\t ');

%%%%%%%%%%%% approximations of the functions u1^2 and u2^2 %%%%%%%%%%%%

fprintf(fw,'\n\t\t\t AD {');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du)
    fprintf(fw,'%c',char(du(i)));
    fprintf(fw,' = f1+f2 <= ');
    fprintf(fw,'%c',char(ru(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end

for i = 1:length(du1)
    fprintf(fw,'%c',char(du1(i)));
    fprintf(fw,' = f1-f2 <= ');
    fprintf(fw,'%c',char(ru1(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end

fprintf(fw,'}');
fprintf(fw,'\n\n\t\t\t DA {');
fprintf(fw,'\n\t\t\t\t ');

fprintf(fw,'fu1_1 = {IF  du1_1         THEN au1_1*(f1+f2)+bu1_1};' );
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du)-1
    fprintf(fw,'%c',fu{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(du(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(du(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(au1(i+1)));
    fprintf(fw,'*(f1+f2)+');
    fprintf(fw,'%c',char(bu1(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%c',fu{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(du(i+1)));
fprintf(fw,'     ');
fprintf(fw,'    THEN ');
fprintf(fw,'%c',char(au1(i+2)));
fprintf(fw,'*(f1+f2)+');
fprintf(fw,'%c',char(bu1(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t ');

fprintf(fw,'fu2_1 = {IF  du2_1         THEN au2_1*(f1-f2)+bu2_1};' );
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du1)-1
    fprintf(fw,'%c',fu1{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(du1(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(du1(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(au2(i+1)));
    fprintf(fw,'*(f1-f2)+');
    fprintf(fw,'%c',char(bu2(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%c',fu1{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(du1(i+1)));
fprintf(fw,'     ');
fprintf(fw,'    THEN ');
fprintf(fw,'%c',char(au2(i+2)));
fprintf(fw,'*(f1-f2)+');
fprintf(fw,'%c',char(bu2(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t }');

fprintf(fw,'\n\t\t\t LINEAR { fu1 = ');
for i = 1:length(fu)
   fprintf(fw,'%c',char(fu(i)));
   if(i ~= length(fu))
      fprintf(fw,' + ');
   end
end
fprintf(fw,';');

fprintf(fw,'\n\t\t\t\t\t  fu2 = ');
for i = 1:length(fu1)
   fprintf(fw,'%c',char(fu1(i)));
   if(i ~= length(fu1))
      fprintf(fw,' + ');
   end
end
fprintf(fw,';');
fprintf(fw,'\n\t\t\t\t\t  ');
fprintf(fw,'fc = 1/4*(fu1-fu2); }');
fprintf(fw, '\nOUTPUT { f1_f2 = fc; }');
fprintf(fw,'\n\t\t\t }');
fprintf(fw,'\n }\n');
fprintf(fw,'\n\n\n');
fclose(fw);

%%%
