function hysdel_Nd(apprx_data, fname)
% Converts the PWA approximation of a function of N variable into HYSDEL

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

fw = fopen(fname,'w');

xfmain_size = apprx_data{2};
xf1 = xfmain_size{1};
xf2 = xfmain_size{2};
params_main = cat(1,apprx_data{2}{:});

a_main = params_main(:,2);
c_main = params_main(:,3);

r_temp = isinf(params_main(:,1));
r_pos = find(r_temp ~= 1);
r_main = params_main(r_pos);

xumain_size = apprx_data{4};
xu1 = xumain_size{1};
xu2 = xumain_size{2};
params_mainu = cat(1,apprx_data{4}{:});

au_main = params_mainu(:,2);
cu_main = params_mainu(:,3);

ru_temp = isinf(params_mainu(:,1));
ru_pos = find(ru_temp ~= 1);
ru_main = params_mainu(ru_pos);

sym_variables = apprx_data{7}{1};

inputs_minbounds = apprx_data{8};
inputs_maxbounds = apprx_data{9};

f_f = {};

for i = 1:length(xfmain_size)
    j = num2str(i);
    f_f{i} = strcat('f',j);
    temp11b = f_f{i};
    if(i ~= length(xfmain_size))
        f_f{i} = strcat(temp11b,'_');
    end
end

f_fmain = strcat(f_f{1},f_f{2});

for i = 3 : length(xfmain_size)
    f_fmain = strcat(f_fmain,f_f{i});
end

f_fmain = strcat('PWA_',f_fmain);

fprintf(fw,'SYSTEM f_approx1x2N {');
fprintf(fw,'\n\t\t');
fprintf(fw,'INTERFACE {');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'INPUT  { ');


for i = 1:length(xfmain_size)
    fprintf(fw,'REAL ');
    fprintf(fw,'%s',sym_variables{i});
    fprintf(fw,'[');
    fprintf(fw,'%.4f',inputs_minbounds(i));
    fprintf(fw,', ');
    fprintf(fw,'%.4f',inputs_maxbounds(i));
    fprintf(fw,'];');
    fprintf(fw,'\n\t\t\t\t\t ');
end
  

fprintf(fw,'}');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'OUTPUT { REAL ');
fprintf(fw,'%s',f_fmain);
fprintf(fw,'; }');
fprintf(fw,'\n\t\t\t');
fprintf(fw,'PARAMETER {');

%%% first function %%%

a = cell(1,length(xfmain_size));
c = cell(1,length(xfmain_size));
r = cell(1,length(xfmain_size));
index = 1;
    for k = 1:length(xfmain_size)
        for i = 1:size(xfmain_size{k},1)
            j = num2str(i);
            sub = num2str(index);
            a{k}{i} = strcat('PWA_a',sub);
            temp11a = a{k}{i};
            a{k}{i} = strcat(temp11a,'_');
            temp12a = a{k}{i};
            a{k}{i} = strcat(temp12a,j); 
            c{k}{i} = strcat('PWA_c',sub);
            temp11b = c{k}{i};
            c{k}{i} = strcat(temp11b,'_');
            temp12b = c{k}{i};
            c{k}{i} = strcat(temp12b,j);
            if(i < size(xfmain_size{k},1))
               r{k}{i} = strcat('PWA_r',sub);
               temp11r = r{k}{i};
               r{k}{i} = strcat(temp11r,'_');
               temp12r = r{k}{i};
               r{k}{i} = strcat(temp12r,j);
            end
        end
        index = index+1;
    end
    fprintf(fw,'\n\t\t\t');
        
    index_a = 0;
    index_c = 0;
    index_r = 0;
    index = 1;
    for k = 1:length(xfmain_size)
        fprintf(fw,'/*approximation parameters of the %d.function */',index);
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xfmain_size{k},1)
            fprintf(fw,'%s',char(a{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',a_main(index_a+1));
            if(i ~= size(xfmain_size{k},1))
               fprintf(fw,', ');
            end
            index_a = index_a+1;
        end
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xfmain_size{k},1)
            fprintf(fw,'%s',char(c{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',c_main(index_c+1));
            if(i ~= size(xfmain_size{k},1))
               fprintf(fw,', ');
            end
            index_c = index_c+1;
        end
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xfmain_size{k},1)-1
            fprintf(fw,'%s',char(r{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',r_main(index_r+1));
            if(i ~= size(xfmain_size{k},1)-1)
               fprintf(fw,', ');
            end
            index_r = index_r+1;
        end        
        fprintf(fw,';');
        fprintf(fw,'\n\n\t\t\t ');
        index = index+1;
    end  

%%%%% parameters for the functions fu1 and fu2 %%%%%%

au = cell(1,2*(length(xfmain_size)-1));
cu = cell(1,2*(length(xfmain_size)-1));
ru = cell(1,2*(length(xfmain_size)-1));
index = 1;
    for k = 1:length(au)
        for i = 1:size(xumain_size{k},1)
            j = num2str(i);
            sub = num2str(index);
            au{k}{i} = strcat('PWA_au',sub);
            temp11a = au{k}{i};
            au{k}{i} = strcat(temp11a,'_');
            temp12a = au{k}{i};
            au{k}{i} = strcat(temp12a,j); 
            cu{k}{i} = strcat('PWA_cu',sub);
            temp11b = cu{k}{i};
            cu{k}{i} = strcat(temp11b,'_');
            temp12b = cu{k}{i};
            cu{k}{i} = strcat(temp12b,j);
            if(i < size(xumain_size{k},1))
               ru{k}{i} = strcat('PWA_ru',sub);
               temp11r = ru{k}{i};
               ru{k}{i} = strcat(temp11r,'_');
               temp12r = ru{k}{i};
               ru{k}{i} = strcat(temp12r,j);
            end
        end
        index = index+1;
    end
    fprintf(fw,'\n\t\t\t');
        
    index_au = 0;
    index_cu = 0;
    index_ru = 0;
    index = 1;
    for k = 1:length(au)
        fprintf(fw,'/*approximation parameters of the %d.function u1^2 */',index);
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xumain_size{k},1)
            fprintf(fw,'%s',char(au{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',au_main(index_au+1));
            if(i ~= size(xumain_size{k},1))
               fprintf(fw,', ');
            end
            index_au = index_au+1;
        end
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xumain_size{k},1)
            fprintf(fw,'%s',char(cu{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',cu_main(index_cu+1));
            if(i ~= size(xumain_size{k},1))
               fprintf(fw,', ');
            end
            index_cu = index_cu+1;
        end
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t');
        fprintf(fw,'REAL ');
        for i = 1:size(xumain_size{k},1)-1
            fprintf(fw,'%s',char(ru{k}{i}));
            fprintf(fw,' = ');
            fprintf(fw,'%.4f',ru_main(index_ru+1));
            if(i ~= size(xumain_size{k},1)-1)
               fprintf(fw,', ');
            end
            index_ru = index_ru+1;
        end        
        fprintf(fw,';');
        fprintf(fw,'\n\n\t\t\t');
        index = index+1;
    end    
   
    
fprintf(fw,'}');
fprintf(fw,'\n\t\t');
fprintf(fw,'}');
fprintf(fw,'\n\t\t');
fprintf(fw,'IMPLEMENTATION {');
fprintf(fw,'\n\t\t\t AUX {');
fprintf(fw,'\n\t\t\t');
fprintf(fw,' /*threshold detectors */');
fprintf(fw,'\n\t\t\t BOOL ');

%%% logical switchers %%%    

d = cell(1,length(xfmain_size)-1);
index = 1;
    for k = 1:length(xfmain_size)
        for i = 1:size(xfmain_size{k},1)-1
            j = num2str(i);
            sub = num2str(index);
            d{k}{i} = strcat('PWA_d',sub);
            temp11a = d{k}{i};
            d{k}{i} = strcat(temp11a,'_');
            temp12a = d{k}{i};
            d{k}{i} = strcat(temp12a,j); 
        end
        index = index+1;
    end
    fprintf(fw,'');
        
    index_d = 0;
    index = 1;
    counter = 1;
    for k = 1:length(xfmain_size)
        for i = 1:size(xfmain_size{k},1)-1
            fprintf(fw,'%s',char(d{k}{i}));
            if(counter ~= sum(apprx_data{5})-length(apprx_data{5}))
               fprintf(fw,', ');
               index_d = index_d+1;
               counter = counter + 1;
            end
        end
        index = index+1;
    end        

    du = cell(1,2*(length(xfmain_size)-1));
    index = 1;
    for k = 1:2*(length(xfmain_size)-1)
        for i = 1:size(xumain_size{k},1)-1
            j = num2str(i);
            sub = num2str(index);
            du{k}{i} = strcat('PWA_du',sub);
            temp11a = du{k}{i};
            du{k}{i} = strcat(temp11a,'_');
            temp12a = du{k}{i};
            du{k}{i} = strcat(temp12a,j); 
        end
        index = index+1;
    end
    fprintf(fw,';');
    index_du = 0;
    index = 1;
    counter = 1;
    fprintf(fw,'\n\t\t\t BOOL ');
    for k = 1:2*(length(xfmain_size)-1)
        for i = 1:size(xumain_size{k},1)-1
            fprintf(fw,'%s',char(du{k}{i}));
            if(counter ~= sum(apprx_data{6})-length(apprx_data{6}))
               fprintf(fw,', ');
               index_du = index_du+1;
            end
            counter = counter + 1;
        end
        index = index+1;
    end     
    fprintf(fw,';');
     
    
%%%% auxiliary variables for original functions %%%%

fprintf(fw,'\n\t\t\t');
fprintf(fw,' /*auxiliary variables */');
fprintf(fw,'\n\t\t\t REAL ');

f = cell(1,length(xfmain_size));
    index = 1;
    for k = 1:length(xfmain_size)
        for i = 1:size(xfmain_size{k},1)
            j = num2str(i);
            sub = num2str(index);
            f{k}{i} = strcat('PWA_f',sub);
            temp11a = f{k}{i};
            f{k}{i} = strcat(temp11a,'_');
            temp12a = f{k}{i};
            f{k}{i} = strcat(temp12a,j); 
        end
        index = index+1;
    end
    index_f = 0;
    index = 1;
    counter = 1;
    for k = 1:length(xfmain_size)
        for i = 1:size(xfmain_size{k},1)
            fprintf(fw,'%s',char(f{k}{i}));
            if(counter ~= sum(apprx_data{5}))
               fprintf(fw,', ');
               index_f = index_f+1;
            end
            counter = counter + 1;
        end
        index = index+1;
    end        
    fprintf(fw,';');
   
%%%% auxiliary variables for u1^2 u2^2 .... %%%%

fprintf(fw,'\n\t\t\t REAL ');

fu = cell(1,2*(length(xfmain_size)-1));
    index = 1;
    for k = 1:2*(length(xfmain_size)-1)
        for i = 1:size(xumain_size{k},1)
            j = num2str(i);
            sub = num2str(index);
            fu{k}{i} = strcat('PWA_fu',sub);
            temp11a = fu{k}{i};
            fu{k}{i} = strcat(temp11a,'_');
            temp12a = fu{k}{i};
            fu{k}{i} = strcat(temp12a,j); 
        end
        index = index+1;
    end
    index_fu = 0;
    index = 1;
    counter = 1;
    for k = 1:2*(length(xfmain_size)-1)
        for i = 1:size(xumain_size{k},1)
            fprintf(fw,'%s',char(fu{k}{i}));
            if(counter ~= sum(apprx_data{6}))
               fprintf(fw,', ');
               index_fu = index_fu+1;
            end
            counter = counter + 1;
        end
        index = index+1;
    end        
    fprintf(fw,';');
        
        
%%%% main auxiliary variables %%%%   

fu_1 = cell(1,2*(length(xfmain_size)-1));
f_c = cell(1,length(xfmain_size)-2);
    for k = 1:2*(length(xfmain_size)-1)
        j = num2str(k);
        f_1{k} = strcat('PWA_f',j);
        fu_1{k} = strcat('PWA_fu',j);
    end


f_1 = cell(1,length(xfmain_size));

    for k = 1:length(xfmain_size)
        j = num2str(k);
        f_1{k} = strcat('PWA_f',j);
    end
        
    fprintf(fw,'\n\t\t\t REAL ');
    for k = 1:length(xfmain_size)
        fprintf(fw,'%s',char(f_1{k}));
        if(k ~= length(xfmain_size))
           fprintf(fw,', '); 
        end
    end
fprintf(fw,';');

fprintf(fw,'\n\t\t\t REAL ');
    for k = 1:2*(length(xfmain_size)-1)
        fprintf(fw,'%s',char(fu_1{k}));
        if(k ~= 2*(length(xfmain_size)-1))
           fprintf(fw,', '); 
        end
    end
fprintf(fw,';');    
    
for k = 1:length(xfmain_size)-1
    j = num2str(k);
    f_c{k} = strcat('PWA_fc',j);
end    

if(isempty(f_c))
    syms fc
    f_c{1} = fc;
end


if(length(f_c) > 1)    
fprintf(fw,'\n\t\t\t REAL ');
    for k = 1:length(xfmain_size)-1
        fprintf(fw,'%s',char(f_c{k}));
        if(k ~= (length(xfmain_size)-1))
           fprintf(fw,', '); 
        end
    end    
else
fprintf(fw,', ');    
fprintf(fw,'%s',char(f_c{1}));
end

fmainForNdim = cell(1,length(f_1)+length(f_c));
fmainForNdim{1} = f_1{1};
fmainForNdim{2} = f_1{2};

index = 1;
index2 = 3;
for h = 3:length(fmainForNdim)
    if(rem(h,2))
        fmainForNdim{h} = f_c{index};
        index = index+1;
    else
        fmainForNdim{h} = f_1{index2};
        index2 = index2 + 1;
    end
end
    
    
fprintf(fw,';');      
fprintf(fw,'\n\t\t\t }');    
fprintf(fw,'\n\t\t\t ');    
fprintf(fw,'AD {');    
fprintf(fw,'\n\t\t\t\t ');

for k = 1:2
    for i = 1:length(d{k})
        fprintf(fw,'%s',char(d{k}(i)));
        fprintf(fw,' = ');
        fprintf(fw,'%s',sym_variables{k});
        fprintf(fw,' <= ');
        fprintf(fw,'%s',char(r{k}(i)));
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t\t ');
    end
end
fprintf(fw,'}');
fprintf(fw,'\n\t\t\t DA {');
fprintf(fw,'\n\t\t\t\t');

if(size(xf1,1) == 2)

    indexf = 1;  
       for t = 1:2        
          fprintf(fw,'%s',char(f{1}(t)));
          if(t == 1)
             fprintf(fw,' = {IF ');
          else
             fprintf(fw,' = {IF ~'); 
          end
          fprintf(fw,'%s',char(d{1}(indexf)));
          if(t == 1)
            fprintf(fw,'           THEN ');
          else
            fprintf(fw,'          THEN ');
          end
          fprintf(fw,'%s',char(a{1}(t)));
          fprintf(fw,'*');
          fprintf(fw,'%s',sym_variables{i});
          fprintf(fw,'+');
          fprintf(fw,'%s',char(c{1}(t)));
          fprintf(fw,'};');
          if(t == 1)
              fprintf(fw,'\n\t\t\t\t ');
          end
       end

else

    fprintf(fw,'%c',f{1}{1});
    fprintf(fw,' = {IF ');
    fprintf(fw,'%c',char(d{1}(1)));
    fprintf(fw,'            THEN ');
    fprintf(fw,'%c',char(a{1}(1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',sym_variables{1});
    fprintf(fw,'+');
    fprintf(fw,'%c',char(c{1}(1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t');   
    
for i = 1:length(d{1})-1
    fprintf(fw,'%c',f{1}{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(d{1}(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(d{1}(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(a{1}(i+1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',sym_variables{1});
    fprintf(fw,'+');
    fprintf(fw,'%c',char(c{1}(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t');
end
fprintf(fw,'%c',f{1}{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(d{1}(i+1)));
fprintf(fw,'     ');
fprintf(fw,'       THEN ');
fprintf(fw,'%c',char(a{1}(i+2)));
fprintf(fw,'*');
fprintf(fw,'%s',sym_variables{1});
fprintf(fw,'+');
fprintf(fw,'%c',char(c{1}(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t');

end

if(size(xf2,1) == 2)
    
  indexf = 1;  
       for t = 1:2 
          if(size(xf1,1) == 2) 
            fprintf(fw,'\n\t\t\t\t ');
          end
          fprintf(fw,'%s',char(f{2}(t)));
          if(t == 1)
             fprintf(fw,' = {IF ');
          else
             fprintf(fw,' = {IF ~'); 
          end
          fprintf(fw,'%s',char(d{2}(indexf)));
          if(t == 1)
            fprintf(fw,'             THEN ');
          else
            fprintf(fw,'            THEN ');
          end
          fprintf(fw,'%s',char(a{2}(t)));
          fprintf(fw,'*');
          fprintf(fw,'%s',sym_variables{2});
          fprintf(fw,'+');
          fprintf(fw,'%s',char(c{2}(t)));
          fprintf(fw,'};');
          if(t == 1)
              fprintf(fw,'\n\t\t\t\t');
          end
       end

else

    fprintf(fw,'%c',f{2}{1});
    fprintf(fw,' = {IF ');
    fprintf(fw,'%c',char(d{2}(1)));
    fprintf(fw,'    THEN ');
    fprintf(fw,'%c',char(a{2}(1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',sym_variables{2});
    fprintf(fw,'+');
    fprintf(fw,'%c',char(c{2}(1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t'); 

for i = 1:length(d{2})-1
    fprintf(fw,'%c',f{2}{i+1});
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%c',char(d{2}(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%c',char(d{2}(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%c',char(a{2}(i+1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',sym_variables{2});
    fprintf(fw,'+');
    fprintf(fw,'%c',char(c{2}(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t');
end
fprintf(fw,'%c',f{2}{i+2});
fprintf(fw,' = {IF ~');
fprintf(fw,'%c',char(d{2}(i+1)));
fprintf(fw,'     ');
fprintf(fw,'     THEN ');
fprintf(fw,'%c',char(a{2}(i+2)));
fprintf(fw,'*');
fprintf(fw,'%s',sym_variables{2});
fprintf(fw,'+');
fprintf(fw,'%c',char(c{2}(i+2)));
fprintf(fw,'};');

end

fprintf(fw,'\n\t\t\t }');

fprintf(fw,'\n\t\t\t LINEAR { ');
for k = 1:2
    fprintf(fw,'%s',char(f_1(k)));
    fprintf(fw,' = ');
    for i = 1:length(f{k})
       fprintf(fw,'%s',char(f{k}(i)));
       if(i ~= length(f{k}))
          fprintf(fw,' + ');
       end
    end
    fprintf(fw,';');
    if(k ~= 2)
        fprintf(fw,'\n\t\t\t\t\t  ');
    end
end
 
% %%%%%%%%%%%% approximations of the functions u1^2 and u2^2 %%%%%%%%%%%%

fprintf(fw,' } ');
fprintf(fw,'\n\t\t\t AD {');
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du{1})
    fprintf(fw,'%c',char(du{1}(i)));
    fprintf(fw,' = PWA_f1+PWA_f2 <= ');
    fprintf(fw,'%c',char(ru{1}(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end

for i = 1:length(du{2})
    fprintf(fw,'%c',char(du{2}(i)));
    fprintf(fw,' = PWA_f1-PWA_f2 <= ');
    fprintf(fw,'%c',char(ru{2}(i)));
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'}');

fprintf(fw,'\n\t\t\t DA {');
fprintf(fw,'\n\t\t\t ');

if(size(xu1,1) == 2)
    
    fprintf(fw,'\t ');
    fprintf(fw,'PWA_fu1_1 = {IF  PWA_du1_1             THEN PWA_au1_1*(PWA_f1+PWA_f2)+PWA_cu1_1};' );
    fprintf(fw,'\n\t\t\t\t ');
    fprintf(fw,'PWA_fu1_2 = {IF  ~PWA_du1_1            THEN PWA_au1_2*(PWA_f1+PWA_f2)+PWA_cu1_2};' );
    fprintf(fw,'\n\t\t\t\t ');

else

fprintf(fw,'\t ');    
fprintf(fw,'PWA_fu1_1 = {IF  PWA_du1_1             THEN PWA_au1_1*(PWA_f1+PWA_f2)+PWA_cu1_1};' );
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du{1})-1
    fprintf(fw,'%s',char(fu{1}(i+1)));
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%s',char(du{1}(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%s',char(du{1}(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%s',char(au{1}(i+1)));
    fprintf(fw,'*(PWA_f1+PWA_f2)+');
    fprintf(fw,'%s',char(cu{1}(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%s',char(fu{1}(i+2)));
fprintf(fw,' = {IF ~');
fprintf(fw,'%s',char(du{1}(i+1)));
fprintf(fw,'     ');
fprintf(fw,'        THEN ');
fprintf(fw,'%s',char(au{1}(i+2)));
fprintf(fw,'*(PWA_f1+PWA_f2)+');
fprintf(fw,'%s',char(cu{1}(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t\t ');

end

if(size(xu2,1) == 2)

    fprintf(fw,'PWA_fu2_1 = {IF  PWA_du2_1             THEN PWA_au2_1*(PWA_f1-PWA_f2)+PWA_cu2_1};' );
    fprintf(fw,'\n\t\t\t\t ');
    fprintf(fw,'PWA_fu2_2 = {IF  ~PWA_du2_1            THEN PWA_au2_2*(PWA_f1-PWA_f2)+PWA_cu2_2};' );
    fprintf(fw,'\n\t\t\t\t ');
    fprintf(fw,'}');
    
else


fprintf(fw,'PWA_fu2_1 = {IF  PWA_du2_1         THEN PWA_au2_1*(PWA_f1-PWA_f2)+PWA_cu2_1};' );
fprintf(fw,'\n\t\t\t\t ');

for i = 1:length(du{2})-1
    fprintf(fw,'%s',char(fu{2}(i+1)));
    fprintf(fw,' = {IF ~');
    fprintf(fw,'%s',char(du{2}(i)));
    fprintf(fw,' & ');
    fprintf(fw,'%s',char(du{2}(i+1)));
    fprintf(fw,' THEN ');
    fprintf(fw,'%s',char(au{2}(i+1)));
    fprintf(fw,'*(PWA_f1-PWA_f2)+');
    fprintf(fw,'%s',char(cu{2}(i+1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
end
fprintf(fw,'%s',char(fu{2}(i+2)));
fprintf(fw,' = {IF ~');
fprintf(fw,'%s',char(du{2}(i+1)));
fprintf(fw,'     ');
fprintf(fw,'    THEN ');
fprintf(fw,'%s',char(au{2}(i+2)));
fprintf(fw,'*(PWA_f1-PWA_f2)+');
fprintf(fw,'%s',char(cu{2}(i+2)));
fprintf(fw,'};');
fprintf(fw,'\n\t\t\t }');

end

fprintf(fw,'\n\t\t\t LINEAR { ');
for k = 1:2
    fprintf(fw,'%s',char(fu_1{k}));
    fprintf(fw,' = ');
    for i = 1:length(fu{k})
       fprintf(fw,'%s',char(fu{k}(i)));
       if(i ~= length(fu{k}))
          fprintf(fw,' + ');
       end
    end
    fprintf(fw,';');
    fprintf(fw,'\n\t\t\t\t\t  ');
end


fprintf(fw,'PWA_fc1 = (PWA_fu1-PWA_fu2); }');


%%% for dimension N > 2

ind = 1;
xstart = 3;

for i = 3:length(xfmain_size)
    
    fprintf(fw,'\n\t\t\t ');
    
    %%%% AD section %%%%
    
    fprintf(fw,'AD {');
    fprintf(fw,'\n\t\t\t\t ');
    for k = 1:length(d{i})
        fprintf(fw,'%s',char(d{i}(k)));
        fprintf(fw,' = ');
        fprintf(fw,'%s',sym_variables{i});
        fprintf(fw,' <= ');
        fprintf(fw,'%s',char(r{i}(k)));
        fprintf(fw,';');
        fprintf(fw,'\n\t\t\t\t ');
    end
    fprintf(fw,'}');
        
    %%%% DA section %%%%
    fprintf(fw,'\n\t\t\t ');
    fprintf(fw,'DA {');
    
    if(size(xfmain_size{i},1) == 2)
       
       indexf = 1;  
       for t = 1:2        
          fprintf(fw,'\n\t\t\t\t ');
          fprintf(fw,'%s',char(f{i}(t)));
          if(t == 1)
             fprintf(fw,' = {IF ');
          else
             fprintf(fw,' = {IF ~'); 
          end
          fprintf(fw,'%s',char(d{i}(indexf)));
          if(t == 1)
            fprintf(fw,'               THEN ');
          else
            fprintf(fw,'              THEN ');
          end
          fprintf(fw,'%s',char(a{i}(t)));
          fprintf(fw,'*');
          fprintf(fw,'%s',sym_variables{i});
          fprintf(fw,' + ');
          fprintf(fw,'%s',char(c{i}(t)));
          fprintf(fw,'};');
       end
    
    else
    
    fprintf(fw,'\n\t\t\t\t ');
    fprintf(fw,'%s',char(f{i}(1)));
    fprintf(fw,' = {IF ');
    fprintf(fw,'%s',char(d{i}(1)));
    fprintf(fw,'              THEN ');
    fprintf(fw,'%s',char(a{i}(1)));
    fprintf(fw,'*');
    fprintf(fw,'%s',sym_variables{i});
    fprintf(fw,' + ');
    fprintf(fw,'%s',char(c{i}(1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');    
        
    for n = 1:length(d{i})-1
        fprintf(fw,'%s',char(f{i}(n+1)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(d{i}(n)));
        fprintf(fw,' & ');
        fprintf(fw,'%s',char(d{i}(n+1)));
        fprintf(fw,'  THEN ');
        fprintf(fw,'%s',char(a{i}(n+1)));
        fprintf(fw,'*');
        fprintf(fw,'%s',sym_variables{i});
        fprintf(fw,' + ');
        fprintf(fw,'%s',char(c{i}(n+1)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');
    end
        fprintf(fw,'%s',char(f{i}(n+2)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(d{i}(n+1)));
        fprintf(fw,'     ');
        fprintf(fw,'        THEN ');
        fprintf(fw,'%s',char(a{i}(n+2)));
        fprintf(fw,'*');
        fprintf(fw,'%s',sym_variables{i});
        fprintf(fw,' + ');
        fprintf(fw,'%s',char(c{i}(n+2)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');
        
    end
    fprintf(fw,'\n\t\t\t }');      
        
    fprintf(fw,'\n\t\t\t LINEAR { ');
     
     %%% Linear section for the functions fi, where i = 3, ... N
     
     fprintf(fw,'%s',char(f_1{i}));
     fprintf(fw,' = ');
     for v = 1:length(f{i})
         fprintf(fw,'%s',char(f{i}(v)));
         if(v ~= length(f{i}))
            fprintf(fw,' + ');
         end
     end
         fprintf(fw,';');
         if(v ~= 2)
%             fprintf(fw,'}'); 
            fprintf(fw,'\n\t\t\t\t\t');
         end  
      
     fprintf(fw,' }');    
         
for ii = xstart:length(xumain_size)
     
     %%%% AD section %%%%
     fprintf(fw,'\n\t\t\t ');    
     fprintf(fw,'AD {');
     fprintf(fw,'\n\t\t\t\t ');
     for g = 1:length(du{ii})
             fprintf(fw,'%s',char(du{ii}(g)));
             fprintf(fw,' = ');
             fprintf(fw,'%s + %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
             fprintf(fw,' <= ');
             fprintf(fw,'%s',char(ru{ii}(g)));
             fprintf(fw,';');
             fprintf(fw,'\n\t\t\t\t ');
     end
        
     for g = 1:length(du{ii+1})
         fprintf(fw,'%s',char(du{ii+1}(g)));
         fprintf(fw,' = ');
         fprintf(fw,'%s - %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
         fprintf(fw,' ');
         fprintf(fw,' <= ');
         fprintf(fw,'%s',char(ru{ii+1}(g)));
         fprintf(fw,';');
         fprintf(fw,'\n\t\t\t\t ');
     end
     fprintf(fw,'}');
        
    
     %%%% DA section %%%%
    fprintf(fw,'\n\t\t\t ');
    fprintf(fw,'DA {');
    
   if(size(xumain_size{ii},1) == 2)
      fprintf(fw,'\n\t\t\t\t ');
      indexu = 1;  
      for t = 1:2        
          fprintf(fw,'%s',char(fu{ii}(t)));
          if(t == 1)
          fprintf(fw,' = {IF ');
          else
          fprintf(fw,' = {IF ~');    
          end
          fprintf(fw,'%s',char(du{ii}(indexu)));
          if(t == 1)
            fprintf(fw,'              THEN ');
          else
            fprintf(fw,'             THEN ');
          end
          fprintf(fw,'%s',char(au{ii}(t)));
          fprintf(fw,'*(');
          fprintf(fw,'%s + %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
          fprintf(fw,') + ');
          fprintf(fw,'%s',char(cu{ii}(t)));
          fprintf(fw,'};');
          if(t == 1)
             fprintf(fw,'\n\t\t\t\t ');
          end
      end
    
    else
    
    fprintf(fw,'\n\t\t\t\t ');
    fprintf(fw,'%s',char(fu{ii}(1)));
    fprintf(fw,' = {IF ');
    fprintf(fw,'%s',char(du{ii}(1)));
    fprintf(fw,'              THEN ');
    fprintf(fw,'%s',char(au{ii}(1)));
    fprintf(fw,'*(');
    fprintf(fw,'%s + %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
    fprintf(fw,') + ');
    fprintf(fw,'%s',char(cu{ii}(1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');

    for n = 1:length(du{ii})-1
        fprintf(fw,'%s',char(fu{ii}(n+1)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(du{ii}(n)));
        fprintf(fw,' & ');
        fprintf(fw,'%s',char(du{ii}(n+1)));
        fprintf(fw,' THEN ');
        fprintf(fw,'%s',char(au{ii}(n+1)));
        fprintf(fw,'*(');
        fprintf(fw,'%s + %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
        fprintf(fw,') + ');
        fprintf(fw,'%s',char(cu{ii}(n+1)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');
    end
        fprintf(fw,'%s',char(fu{ii}(n+2)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(du{ii}(n+1)));
        fprintf(fw,'     ');
        fprintf(fw,'        THEN ');
        fprintf(fw,'%s',char(au{ii}(n+2)));
        fprintf(fw,'*(');
        fprintf(fw,'%s + %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
        fprintf(fw,') + ');
        fprintf(fw,'%s',char(cu{ii}(n+2)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');
        
   end   
    
    if(size(xumain_size{ii+1},1) == 2)
       if(size(xumain_size{ii},1) == 2)
          fprintf(fw,'\n\t\t\t\t '); 
       end
       indexu2 = 1; 
       for t = 1:2        
          fprintf(fw,'%s',char(fu{ii+1}(t)));
          if(t == 1)
             fprintf(fw,' = {IF ');
          else
             fprintf(fw,' = {IF ~'); 
          end
          fprintf(fw,'%s',char(du{ii+1}(indexu2)));
          if(t == 1)
             fprintf(fw,'              THEN ');
          else
             fprintf(fw,'             THEN ');
          end
          fprintf(fw,'%s',char(au{ii+1}(t)));
          fprintf(fw,'*(');
          fprintf(fw,'%s - %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
          fprintf(fw,') + ');
          fprintf(fw,'%s',char(cu{ii+1}(t)));
          fprintf(fw,'};');
          if(t == 1)
            fprintf(fw,'\n\t\t\t\t ');
          end
       end
    
    else 
    if(size(xumain_size{ii},1) == 2)    
       fprintf(fw,'\n\t\t\t\t ');
    end
    fprintf(fw,'%s',char(fu{ii+1}(1)));
    fprintf(fw,' = {IF ');
    fprintf(fw,'%s',char(du{ii+1}(1)));
    fprintf(fw,'              THEN ');
    fprintf(fw,'%s',char(au{ii+1}(1)));
    fprintf(fw,'*(');
    fprintf(fw,'%s - %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
    fprintf(fw,') + ');
    fprintf(fw,'%s',char(cu{ii+1}(1)));
    fprintf(fw,'};');
    fprintf(fw,'\n\t\t\t\t ');
      
    for n = 1:length(du{ii+1})-1
        fprintf(fw,'%s',char(fu{ii+1}(n+1)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(du{ii+1}(n)));
        fprintf(fw,' & ');
        fprintf(fw,'%s',char(du{ii+1}(n+1)));
        fprintf(fw,' THEN ');
        fprintf(fw,'%s',char(au{ii+1}(n+1)));
        fprintf(fw,'*(');
        fprintf(fw,'%s - %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
        fprintf(fw,') + ');
        fprintf(fw,'%s',char(cu{ii+1}(n+1)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');
    end
        fprintf(fw,'%s',char(fu{ii+1}(n+2)));
        fprintf(fw,' = {IF ~');
        fprintf(fw,'%s',char(du{ii+1}(n+1)));
        fprintf(fw,'     ');
        fprintf(fw,'        THEN ');
        fprintf(fw,'%s',char(au{ii+1}(n+2)));
        fprintf(fw,'*(');
        fprintf(fw,'%s - %s',char(fmainForNdim{ii}),char(fmainForNdim{ii+1}));
        fprintf(fw,') + ');
        fprintf(fw,'%s',char(cu{ii+1}(n+2)));
        fprintf(fw,'};');
        fprintf(fw,'\n\t\t\t\t ');   
       
    end   
     
     fprintf(fw,'\n\t\t\t ');
     fprintf(fw,'}');   
          
     %%% Linear section for the functions fi, where i = 3, ... N
     
     fprintf(fw,'\n\t\t\t LINEAR { ');
     for k = ii:ii+1
         fprintf(fw,'%s',char(fu_1{k}));
         fprintf(fw,' = ');
         for t = 1:size(xumain_size{k},1)
             fprintf(fw,'%s',char(fu{k}(t)));
                 if(t ~= size(xumain_size{k},1))
                    fprintf(fw,' + ');
                 end
         end
             fprintf(fw,';');
             fprintf(fw,'\n\t\t\t\t\t  ');
     end
         fprintf(fw,'%s',char(f_c{ind+1}));
         fprintf(fw,' = (');
         fprintf(fw,'%s - %s',char(fu_1{k-1}),char(fu_1{k}));
         fprintf(fw,');');
         fprintf(fw,' }');
         
            
         xstart = xstart+2;
         ind = ind+1;
         break;
    end
   
end
fprintf(fw,'\n ');
fprintf(fw,'OUTPUT {');
fprintf(fw,'%s',char(f_fmain));
fprintf(fw,' = ');
fprintf(fw,'%s',char(f_c{ind}));
fprintf(fw,';}');
fprintf(fw,'\n\t\t }');
fprintf(fw,'\n\t }');

