function Z = points2regions(xs, f)
% Converts linearization points into regions, slopes and offsets

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

syms x
fds = diff(f(x), x);
fd = matlabFunction(fds);

ns = length(xs);

if nargin(fd)==0
    a = ones(ns, 1)*double(fds);
    b = zeros(ns, 1);
    intersections = xs(2:end);
else
    a = fd(xs);
    b = f(xs)-fd(xs).*xs;
    intersections = zeros(ns-1, 1);
    for i = 1:ns-1
        intersections(i) = ( b(i+1) - b(i) ) / ( a(i) - a(i+1) );
    end
end
Z = [[intersections(:); Inf] a(:) b(:)];
