function [subL,subV] = separate_functions(L,V)
% Split the separable functions into a sequence of 1D functions

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------

%%% memory allocation
subL = cell(1,length(L));
subV = cell(1,length(V));

%%% counting the number of signs(*)
for k = 1:length(L)
    
    %%% converting the symbolic fuction to a string %%%
    Fpom = char(L{k});
    
    %%% counting the signs (*) %%%x
    resultPozChar = findstr(Fpom,'*');
    resultPozChar1 = findstr(V{k},',');
    
    %%% memory allocation for each subfunction %%%
    subL{k} = cell(1,length(resultPozChar)+1);
    subV{k} = cell(1,length(resultPozChar1)+1);

    %%% length(final_array_of_signs) >= 2
    if(length(resultPozChar) >= 2)
       subL{k}{1} = Fpom((1:resultPozChar(1)-1));
       for i = 1:length(resultPozChar)-1
           subL{k}{i+1} = Fpom(resultPozChar(i)+1:resultPozChar(i+1)-1);
       end
       subL{k}{end} = Fpom(resultPozChar(i+1)+1:length(Fpom));
    %%% special case: length(final_array_of_signs) = 1
    elseif(length(resultPozChar) == 1)
       subL{k}{1} = Fpom(1:resultPozChar(1)-1); 
       subL{k}{2} = Fpom(resultPozChar(1)+1:length(Fpom));
    else
    %%% special case: length(final_array_of_signs) = 0
       subL{k}{1} = Fpom;
    end
    
    Fpom = V{k};
    
    %%% length(final_array_of_signs) >= 2
    if(length(resultPozChar1) >= 2)
       subV{k}{1} = Fpom((1:resultPozChar1(1)-1));
       for i = 1:length(resultPozChar1)-1
           subV{k}{i+1} = Fpom(resultPozChar1(i)+1:resultPozChar1(i+1)-1);
       end
       subV{k}{end} = Fpom(resultPozChar1(i+1)+1:length(Fpom));
    %%% special case: length(final_array_of_signs) = 1
    elseif(length(resultPozChar1) == 1)
       subV{k}{1} = Fpom(1:resultPozChar1(1)-1); 
       subV{k}{2} = Fpom(resultPozChar1(1)+1:length(Fpom));
    else
    %%% special case: length(final_array_of_signs) = 0
       subV{k}{1} = Fpom;
    end

end




