function [L,V] = split(fun,vars)
% Split the original function into a series of separable functions

% Copyright is with the following author(s):
%
% (C) 2010 Alexander Szucs, Slovak University of Technology in Bratislava
%          alexander.szucs@stuba.sk
% (C) 2010 Michal Kvasnica, Slovak University of Technology in Bratislava
%          michal.kvasnica@stuba.sk

% ---------------------------------------------------------------------------
% Legal note:
%          This program is free software; you can redistribute it and/or
%          modify it under the terms of the GNU General Public
%          License as published by the Free Software Foundation; either
%          version 2.1 of the License, or (at your option) any later version.
%
%          This program is distributed in the hope that it will be useful,
%          but WITHOUT ANY WARRANTY; without even the implied warranty of
%          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%          General Public License for more details.
% 
%          You should have received a copy of the GNU General Public
%          License along with this library; if not, write to the 
%          Free Software Foundation, Inc., 
%          59 Temple Place, Suite 330, 
%          Boston, MA  02111-1307  USA
%
% -------------------------------------------------------------------------


%%% convert the symbolic form of the function to string

Fpom = fun;

%%% checking whether the argument contains a minus sign: e.g: exp(-9850/t
%%% +273.15) %%%

brackets = findstr(Fpom,'(');
minus = findstr(Fpom,'-');    
 
pos_minus = []; 
for i = 1:length(brackets)
    for j = 1:length(minus)  
        if(abs(brackets(i) - minus(j)) == 1 & brackets(i) < minus(j))
            pos_minus = [pos_minus; minus(j)];
        end
    end
end

%%% counting the number of signs(+,-)

pozchar_plus = findstr(Fpom,'+');
pozchar_minus = findstr(Fpom,'-');

for i = 1:length(pozchar_minus)
    for j = 1:length(pos_minus)
        if(pozchar_minus(i) == pos_minus(j))
            pozchar_minus(i) = 0;
        end
    end
end

pos = find(pozchar_minus ~= 0);
pozchar_minus = pozchar_minus(pos);

%%% final array of signs %%%

resultPozChar = sort([pozchar_plus pozchar_minus]);
L = cell(1,length(resultPozChar)+1);

%%% length(final_array_of_signs) >= 2
if(length(resultPozChar) >= 2)
   L{1} = Fpom((1:resultPozChar(1)-1));
   for i = 1:length(resultPozChar)-1
       L{i+1} = Fpom(resultPozChar(i)+1:resultPozChar(i+1)-1);
   end
   L{end} = Fpom(resultPozChar(i+1)+1:length(Fpom));
%%% special case: length(final_array_of_signs) = 1
elseif(length(resultPozChar) == 1)
   L{1} = Fpom(1:resultPozChar(1)-1); 
   L{2} = Fpom(resultPozChar(1)+1:length(Fpom));
else
%%% special case: length(final_array_of_signs) = 0
   L{1} = Fpom;
end

%%% memory allocation %%%
V = cell(1,length(L));

%%% obtaining the smbolic variables %%%
for i = 1:length(L)
    V{i} = findsym(sym(L{i}));
end


    

